@@ -1,5 +1,9 @@
 # -*- coding: utf-8 -*-
 
+import os
+import sys
+import copy
+
 from jinja2 import Environment
 from jinja2 import Template
 from jinja2 import loaders
@@ -14,9 +18,9 @@ from django.template import InvalidTemplateLibrary
 from django.template.loaders import app_directories
 from django.utils.importlib import import_module
 
-import os
-import sys
-import copy
+from . import builtins
+from .library import Library
+
 
 JINJA2_ENVIRONMENT_OPTIONS = getattr(settings, 'JINJA2_ENVIRONMENT_OPTIONS', {})
 JINJA2_EXTENSIONS = getattr(settings, 'JINJA2_EXTENSIONS', [])
@@ -25,7 +29,6 @@ JINJA2_TESTS = getattr(settings, 'JINJA2_TESTS', {})
 JINJA2_GLOBALS = getattr(settings, 'JINJA2_GLOBALS', {})
 JINJA2_AUTOESCAPE = getattr(settings, 'JINJA2_AUTOESCAPE', False)
 
-from django_jinja import builtins
 
 JINJA2_FILTERS.update({
     'reverseurl': builtins.filters.reverse,
@@ -193,75 +196,10 @@ class Environment(Environment):
                     else:
                         safestring.SafeBytes.__html__ = lambda self: str(self)
 
-class Library(object):
-    instance = None
-
-    _globals = {}
-    _tests = {}
-    _filters = {}
-
-    def __new__(cls, *args, **kwargs):
-        if cls.instance == None:
-            cls.instance = super(Library, cls).__new__(cls, *args, **kwargs)
-        return cls.instance
-
-    @classmethod
-    def get_instance(cls):
-        return cls.instance
-
-    def _update_env(self, env):
-        env.filters.update(self._filters)
-        env.globals.update(self._globals)
-        env.tests.update(self._tests)
-
-    def _new_function(self, attr, func, name=None):
-        _attr = getattr(self, attr)
-        if name is None:
-            name = func.__name__
-
-        _attr[name] = func
-        return func
-
-    def _function(self, attr, name=None, _function=None):
-        if name is None and _function is None:
-            def dec(func):
-                return self._new_function(attr, func)
-            return dec
-
-        elif name is not None and _function is None:
-            if callable(name):
-                return self._new_function(attr, name)
-
-            else:
-                def dec(func):
-                    return self._function(attr, name, func)
-
-                return dec
-
-        elif name is not None and _function is not None:
-            return self._new_function(attr, _function, name)
-
-        raise RuntimeError("Invalid parameters")
-
-    def global_function(self, *args, **kwargs):
-        return self._function("_globals", *args, **kwargs)
-
-    def test(self, *args, **kwargs):
-        return self._function("_tests", *args, **kwargs)
-
-    def filter(self, *args, **kwargs):
-        return self._function("_filters", *args, **kwargs)
-
-    def __setitem__(self, item, value):
-        self.globals[item] = value
-
-    def __getitem__(self, item, value): #for reciprocity with __setitem__
-        return self.globals[item]
-
 
 initial_params = {
     'autoescape': JINJA2_AUTOESCAPE,
-    'loader': FileSystemLoader(app_directories.app_template_dirs + settings.TEMPLATE_DIRS),
+    'loader': FileSystemLoader(app_directories.app_template_dirs + tuple(settings.TEMPLATE_DIRS)),
     'extensions': JINJA2_EXTENSIONS + ['jinja2.ext.i18n', 'jinja2.ext.autoescape'],
 }
 
